package com.iterationtwo.boe.dao;

import java.sql.*;
import java.util.*;
import java.text.*;
import com.iterationtwo.boe.vo.*;

public class ApplicationDAO
{
    public static ApplicationDAO getInstance()
    {
      if ( dao == null )
        dao = new ApplicationDAO();

      return dao;

    }

//------------------------------------------------------------------------------

    private ApplicationDAO()
    {
      try
      {
        Class.forName( "org.gjt.mm.mysql.Driver" );
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

//------------------------------------------------------------------------------

    public List getApplications() throws SQLException
    {
    	Connection con = null;
        List results = new ArrayList();
        try
        {
            con = DriverManager.getConnection( url );

            PreparedStatement stmt = con.prepareStatement( "SELECT * FROM applications" );

            ResultSet rs = stmt.executeQuery();
            while (rs.next())
            {
                ApplicationVO app = new ApplicationVO();
                app.setId( rs.getInt( "id" ) );
                app.setFirstName( rs.getString( "first_name" ) );
                app.setLastName( rs.getString( "last_name" ) );
                app.setAddressLine1( rs.getString( "address_line_1" ) );
                app.setAddressLine2( rs.getString( "address_line_2" ) );
                app.setAddressLine3( rs.getString( "address_line_3" ) );
                app.setCity( rs.getString( "city" ) );
                app.setCountry( rs.getString( "country" ) );
                app.setPostcode( rs.getString( "postcode" ) );
                app.setDateOfBirth( rs.getTimestamp( "date_of_birth" ) );
                app.setCreditCardId( rs.getInt( "credit_card_id" ) );
                app.setStatus( rs.getInt( "status" ) );
                app.setGenderIsMale( rs.getBoolean( "gender_is_male" ) );
                app.setCreditLimit( rs.getInt( "credit_limit" ) );
                app.setRequestedCreditLimit( rs.getInt( "requested_credit_limit" ) );
                app.setDateOfApplication( rs.getTimestamp( "date_of_application" ) );
                app.setLostOrStolen( rs.getBoolean( "lost_or_stolen" ) );

                results.add( app );
            }
            return results;
        }
        finally
        {
            try
            {
                con.close();
            }
            catch (Exception ignored) {}
        }
    }

  //------------------------------------------------------------------------------

      public List getApplicationsOverTime( int creditCardId, int monthsBack ) throws SQLException
      {
          Connection con = null;

          SimpleDateFormat df = new SimpleDateFormat( "MMM-yyyy" );

          List results = new ArrayList( monthsBack );
          for ( int i = 0; i <= monthsBack; i++ )
          {
            results.add( new CountVO() );
          }

          try
          {
              con = DriverManager.getConnection( url );

              for ( int i = 0; i <= monthsBack; i++ )
              {
                PreparedStatement stmt = con.prepareStatement("select count(*) AS c from applications where credit_card_id = ? AND date_of_application >= ? AND date_of_application < ?");
                stmt.setInt( 1, creditCardId );

                java.util.Date now = new java.util.Date();

                Calendar startDate = GregorianCalendar.getInstance();
                startDate.add( Calendar.MONTH, -i );
                startDate.set( Calendar.DATE, 1 );
                startDate.set( Calendar.HOUR_OF_DAY, 0 );
                startDate.set( Calendar.MINUTE, 0 );
                startDate.set( Calendar.SECOND, 0 );

                Calendar endDate = GregorianCalendar.getInstance();
                endDate.add( Calendar.MONTH, -i + 1 );
                endDate.set( Calendar.DATE, 1 );
                endDate.set( Calendar.HOUR_OF_DAY, 0 );
                endDate.set( Calendar.MINUTE, 0 );
                endDate.set( Calendar.SECOND, 0 );

                stmt.setTimestamp( 2, new Timestamp( startDate.getTime().getTime() ) );
                stmt.setTimestamp( 3, new Timestamp( endDate.getTime().getTime() ) );

                ResultSet rs = stmt.executeQuery();
                rs.next();

                CountVO count = new CountVO();
                count.setId( monthsBack - i );
                count.setSales( rs.getInt( "c" ) );
                count.setSummary( df.format( startDate.getTime() ) );
                results.set( monthsBack - i, count );
              }
              return results;
          }
          finally
          {
              try
              {
                  con.close();
              }
              catch (Exception ignored) {}
          }
      }

//------------------------------------------------------------------------------

    public int addApplication( ApplicationVO app ) throws SQLException
    {
        Connection con = null;
        List results = new ArrayList();
        try
        {
            con = DriverManager.getConnection( url );

            PreparedStatement stmt = con.prepareStatement( "INSERT INTO applications ( first_name,last_name,address_line_1,address_line_2,address_line_3,city,country,postcode,date_of_birth,credit_card_id,status,gender_is_male,credit_limit,requested_credit_limit, date_of_application, lost_or_stolen ) VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), ? )" );

            stmt.setString( 1, app.getFirstName() );
            stmt.setString( 2, app.getLastName() );
            stmt.setString( 3, app.getAddressLine1() );
            stmt.setString( 4, app.getAddressLine2() );
            stmt.setString( 5, app.getAddressLine3() );
            stmt.setString( 6, app.getCity() );
            stmt.setString( 7, app.getCountry() );
            stmt.setString( 8, app.getPostcode() );
            stmt.setTimestamp( 9, new java.sql.Timestamp( app.getDateOfBirth().getTime() ) );
            stmt.setInt( 10, app.getCreditCardId() );
            stmt.setInt( 11, app.getStatus() );
            stmt.setBoolean( 12, app.isGenderIsMale() );
            stmt.setInt( 13, app.getCreditLimit() );
            stmt.setInt( 14, app.getRequestedCreditLimit() );
            stmt.setBoolean( 15, app.isLostOrStolen() );

            return stmt.executeUpdate();
        }
        finally
        {
            try
            {
                con.close();
            }
            catch (Exception ignored) {}
        }
    }

  //------------------------------------------------------------------------------

      public List getApplicationCountPerCard() throws SQLException
      {
          Connection con = null;
          List results = new ArrayList();
          try
          {
              con = DriverManager.getConnection( url );

              PreparedStatement stmt = con.prepareStatement( "SELECT credit_cards.id, summary, count(*) AS c FROM applications, credit_cards WHERE credit_cards.id=credit_card_id GROUP BY credit_card_id ORDER BY c DESC" );

              List counts = new ArrayList();

              ResultSet resultSet = stmt.executeQuery();
              while ( resultSet.next() )
              {
                CountVO count = new CountVO();
                count.setId( resultSet.getInt( "id" ) );
                count.setSummary( resultSet.getString( "summary" ) );
                count.setSales( resultSet.getInt( "c" ) );

                counts.add( count );
              }
              return counts;
          }
          finally
          {
              try
              {
                  con.close();
              }
              catch (Exception ignored) {}
          }
      }

  //------------------------------------------------------------------------------

      public void updateApplication( int id, int creditLimit, int status ) throws SQLException
      {
          Connection con = null;
          List results = new ArrayList();
          try
          {
              con = DriverManager.getConnection( url );

              PreparedStatement stmt = con.prepareStatement( "UPDATE applications SET status = ?,credit_limit = ? WHERE id = ?" );

              stmt.setInt( 1, status );
              stmt.setInt( 2, creditLimit );
              stmt.setInt( 3, id );

              stmt.executeUpdate();
          }
          finally
          {
              try
              {
                  con.close();
              }
              catch (Exception ignored) {}
          }
      }

//------------------------------------------------------------------------------

    private static ApplicationDAO dao;
    private static String url = "jdbc:mysql://localhost:3306/flexbook?autoReconnect=true";
}
